/*
 * Copyright 2019 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.android.overtheair.spi.v1.stub;

import com.google.android.overtheair.v1.Chart;
import com.google.android.overtheair.v1.Config;
import com.google.android.overtheair.v1.CreateConfigRequest;
import com.google.android.overtheair.v1.CreateGroupMemberRequest;
import com.google.android.overtheair.v1.CreateGroupMembersRequest;
import com.google.android.overtheair.v1.CreateGroupMembersResponse;
import com.google.android.overtheair.v1.CreateGroupRequest;
import com.google.android.overtheair.v1.DeleteConfigRequest;
import com.google.android.overtheair.v1.DeleteConfigResponse;
import com.google.android.overtheair.v1.DeleteGroupMemberRequest;
import com.google.android.overtheair.v1.DeleteGroupMemberResponse;
import com.google.android.overtheair.v1.DeleteGroupMembersRequest;
import com.google.android.overtheair.v1.DeleteGroupMembersResponse;
import com.google.android.overtheair.v1.DeleteGroupRequest;
import com.google.android.overtheair.v1.DeleteGroupResponse;
import com.google.android.overtheair.v1.DeletePackageRequest;
import com.google.android.overtheair.v1.DeletePackageResponse;
import com.google.android.overtheair.v1.Deployment;
import com.google.android.overtheair.v1.DisableAllEnabledGroupsOfConfigRequest;
import com.google.android.overtheair.v1.DisableAllEnabledGroupsOfConfigResponse;
import com.google.android.overtheair.v1.DisableGroupsRequest;
import com.google.android.overtheair.v1.DisableGroupsResponse;
import com.google.android.overtheair.v1.EnableGroupsRequest;
import com.google.android.overtheair.v1.EnableGroupsResponse;
import com.google.android.overtheair.v1.GetConfigRequest;
import com.google.android.overtheair.v1.GetDeploymentRequest;
import com.google.android.overtheair.v1.GetGroupChartRequest;
import com.google.android.overtheair.v1.GetGroupMemberRequest;
import com.google.android.overtheair.v1.GetGroupRequest;
import com.google.android.overtheair.v1.GetPackageRequest;
import com.google.android.overtheair.v1.Group;
import com.google.android.overtheair.v1.GroupMember;
import com.google.android.overtheair.v1.ListConfigPackagesRequest;
import com.google.android.overtheair.v1.ListConfigPackagesResponse;
import com.google.android.overtheair.v1.ListConfigsRequest;
import com.google.android.overtheair.v1.ListConfigsResponse;
import com.google.android.overtheair.v1.ListConfigsUsingPackageRequest;
import com.google.android.overtheair.v1.ListConfigsUsingPackageResponse;
import com.google.android.overtheair.v1.ListDeploymentsRequest;
import com.google.android.overtheair.v1.ListDeploymentsResponse;
import com.google.android.overtheair.v1.ListGroupChartsRequest;
import com.google.android.overtheair.v1.ListGroupChartsResponse;
import com.google.android.overtheair.v1.ListGroupConfigsRequest;
import com.google.android.overtheair.v1.ListGroupConfigsResponse;
import com.google.android.overtheair.v1.ListGroupMembersRequest;
import com.google.android.overtheair.v1.ListGroupMembersResponse;
import com.google.android.overtheair.v1.ListGroupsRequest;
import com.google.android.overtheair.v1.ListGroupsResponse;
import com.google.android.overtheair.v1.ListPackagesRequest;
import com.google.android.overtheair.v1.ListPackagesResponse;
import com.google.android.overtheair.v1.UpdateConfigPackagesRequest;
import com.google.android.overtheair.v1.UpdateConfigPackagesResponse;
import com.google.android.overtheair.v1.UpdateConfigRequest;
import com.google.android.overtheair.v1.UpdateDeploymentRequest;
import com.google.android.overtheair.v1.UpdateGroupConfigsRequest;
import com.google.android.overtheair.v1.UpdateGroupConfigsResponse;
import com.google.android.overtheair.v1.UpdateGroupRequest;
import com.google.android.overtheair.v1.UpdatePackageRequest;
import com.google.api.core.ApiFunction;
import com.google.api.core.BetaApi;
import com.google.api.gax.core.GaxProperties;
import com.google.api.gax.core.GoogleCredentialsProvider;
import com.google.api.gax.core.InstantiatingExecutorProvider;
import com.google.api.gax.grpc.GaxGrpcProperties;
import com.google.api.gax.grpc.GrpcTransportChannel;
import com.google.api.gax.grpc.InstantiatingGrpcChannelProvider;
import com.google.api.gax.retrying.RetrySettings;
import com.google.api.gax.rpc.ApiClientHeaderProvider;
import com.google.api.gax.rpc.ClientContext;
import com.google.api.gax.rpc.StatusCode;
import com.google.api.gax.rpc.StubSettings;
import com.google.api.gax.rpc.TransportChannelProvider;
import com.google.api.gax.rpc.UnaryCallSettings;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Lists;
import java.io.IOException;
import java.util.List;
import javax.annotation.Generated;
import org.threeten.bp.Duration;

// AUTO-GENERATED DOCUMENTATION AND CLASS
/**
 * Settings class to configure an instance of {@link OverTheAirServiceStub}.
 *
 * <p>The default instance has everything set to sensible defaults:
 *
 * <ul>
 *   <li>The default service address (androidovertheair.googleapis.com) and default port (443) are
 *       used.
 *   <li>Credentials are acquired automatically through Application Default Credentials.
 *   <li>Retries are configured for idempotent methods but not for non-idempotent methods.
 * </ul>
 *
 * <p>The builder of this class is recursive, so contained classes are themselves builders. When
 * build() is called, the tree of builders is called to create the complete settings object. For
 * example, to set the total timeout of getDeployment to 30 seconds:
 *
 * <pre>
 * <code>
 * OverTheAirServiceStubSettings.Builder overTheAirServiceSettingsBuilder =
 *     OverTheAirServiceStubSettings.newBuilder();
 * overTheAirServiceSettingsBuilder.getDeploymentSettings().getRetrySettings().toBuilder()
 *     .setTotalTimeout(Duration.ofSeconds(30));
 * OverTheAirServiceStubSettings overTheAirServiceSettings = overTheAirServiceSettingsBuilder.build();
 * </code>
 * </pre>
 */
@Generated("by gapic-generator")
@BetaApi
public class OverTheAirServiceStubSettings extends StubSettings<OverTheAirServiceStubSettings> {
  /** The default scopes of the service. */
  private static final ImmutableList<String> DEFAULT_SERVICE_SCOPES =
      ImmutableList.<String>builder()
          .add("https://www.googleapis.com/auth/android_partner_over_the_air")
          .add("https://www.googleapis.com/auth/userinfo.email")
          .build();

  private final UnaryCallSettings<GetDeploymentRequest, Deployment> getDeploymentSettings;
  private final UnaryCallSettings<ListDeploymentsRequest, ListDeploymentsResponse>
      listDeploymentsSettings;
  private final UnaryCallSettings<UpdateDeploymentRequest, Deployment> updateDeploymentSettings;
  private final UnaryCallSettings<GetPackageRequest, com.google.android.overtheair.v1.Package>
      getPackageSettings;
  private final UnaryCallSettings<ListPackagesRequest, ListPackagesResponse> listPackagesSettings;
  private final UnaryCallSettings<DeletePackageRequest, DeletePackageResponse>
      deletePackageSettings;
  private final UnaryCallSettings<UpdatePackageRequest, com.google.android.overtheair.v1.Package>
      updatePackageSettings;
  private final UnaryCallSettings<CreateConfigRequest, Config> createConfigSettings;
  private final UnaryCallSettings<GetConfigRequest, Config> getConfigSettings;
  private final UnaryCallSettings<ListConfigsRequest, ListConfigsResponse> listConfigsSettings;
  private final UnaryCallSettings<DeleteConfigRequest, DeleteConfigResponse> deleteConfigSettings;
  private final UnaryCallSettings<UpdateConfigRequest, Config> updateConfigSettings;
  private final UnaryCallSettings<CreateGroupRequest, Group> createGroupSettings;
  private final UnaryCallSettings<GetGroupRequest, Group> getGroupSettings;
  private final UnaryCallSettings<ListGroupsRequest, ListGroupsResponse> listGroupsSettings;
  private final UnaryCallSettings<DeleteGroupRequest, DeleteGroupResponse> deleteGroupSettings;
  private final UnaryCallSettings<UpdateGroupRequest, Group> updateGroupSettings;
  private final UnaryCallSettings<DisableGroupsRequest, DisableGroupsResponse>
      disableGroupsSettings;
  private final UnaryCallSettings<EnableGroupsRequest, EnableGroupsResponse> enableGroupsSettings;
  private final UnaryCallSettings<CreateGroupMemberRequest, GroupMember> createGroupMemberSettings;
  private final UnaryCallSettings<CreateGroupMembersRequest, CreateGroupMembersResponse>
      createGroupMembersSettings;
  private final UnaryCallSettings<GetGroupMemberRequest, GroupMember> getGroupMemberSettings;
  private final UnaryCallSettings<ListGroupMembersRequest, ListGroupMembersResponse>
      listGroupMembersSettings;
  private final UnaryCallSettings<DeleteGroupMemberRequest, DeleteGroupMemberResponse>
      deleteGroupMemberSettings;
  private final UnaryCallSettings<DeleteGroupMembersRequest, DeleteGroupMembersResponse>
      deleteGroupMembersSettings;
  private final UnaryCallSettings<GetGroupChartRequest, Chart> getGroupChartSettings;
  private final UnaryCallSettings<ListGroupChartsRequest, ListGroupChartsResponse>
      listGroupChartsSettings;
  private final UnaryCallSettings<ListGroupConfigsRequest, ListGroupConfigsResponse>
      listGroupConfigsSettings;
  private final UnaryCallSettings<UpdateGroupConfigsRequest, UpdateGroupConfigsResponse>
      updateGroupConfigsSettings;
  private final UnaryCallSettings<
          DisableAllEnabledGroupsOfConfigRequest, DisableAllEnabledGroupsOfConfigResponse>
      disableAllEnabledGroupsOfConfigSettings;
  private final UnaryCallSettings<ListConfigPackagesRequest, ListConfigPackagesResponse>
      listConfigPackagesSettings;
  private final UnaryCallSettings<UpdateConfigPackagesRequest, UpdateConfigPackagesResponse>
      updateConfigPackagesSettings;
  private final UnaryCallSettings<ListConfigsUsingPackageRequest, ListConfigsUsingPackageResponse>
      listConfigsUsingPackageSettings;

  /** Returns the object with the settings used for calls to getDeployment. */
  public UnaryCallSettings<GetDeploymentRequest, Deployment> getDeploymentSettings() {
    return getDeploymentSettings;
  }

  /** Returns the object with the settings used for calls to listDeployments. */
  public UnaryCallSettings<ListDeploymentsRequest, ListDeploymentsResponse>
      listDeploymentsSettings() {
    return listDeploymentsSettings;
  }

  /** Returns the object with the settings used for calls to updateDeployment. */
  public UnaryCallSettings<UpdateDeploymentRequest, Deployment> updateDeploymentSettings() {
    return updateDeploymentSettings;
  }

  /** Returns the object with the settings used for calls to getPackage. */
  public UnaryCallSettings<GetPackageRequest, com.google.android.overtheair.v1.Package>
      getPackageSettings() {
    return getPackageSettings;
  }

  /** Returns the object with the settings used for calls to listPackages. */
  public UnaryCallSettings<ListPackagesRequest, ListPackagesResponse> listPackagesSettings() {
    return listPackagesSettings;
  }

  /** Returns the object with the settings used for calls to deletePackage. */
  public UnaryCallSettings<DeletePackageRequest, DeletePackageResponse> deletePackageSettings() {
    return deletePackageSettings;
  }

  /** Returns the object with the settings used for calls to updatePackage. */
  public UnaryCallSettings<UpdatePackageRequest, com.google.android.overtheair.v1.Package>
      updatePackageSettings() {
    return updatePackageSettings;
  }

  /** Returns the object with the settings used for calls to createConfig. */
  public UnaryCallSettings<CreateConfigRequest, Config> createConfigSettings() {
    return createConfigSettings;
  }

  /** Returns the object with the settings used for calls to getConfig. */
  public UnaryCallSettings<GetConfigRequest, Config> getConfigSettings() {
    return getConfigSettings;
  }

  /** Returns the object with the settings used for calls to listConfigs. */
  public UnaryCallSettings<ListConfigsRequest, ListConfigsResponse> listConfigsSettings() {
    return listConfigsSettings;
  }

  /** Returns the object with the settings used for calls to deleteConfig. */
  public UnaryCallSettings<DeleteConfigRequest, DeleteConfigResponse> deleteConfigSettings() {
    return deleteConfigSettings;
  }

  /** Returns the object with the settings used for calls to updateConfig. */
  public UnaryCallSettings<UpdateConfigRequest, Config> updateConfigSettings() {
    return updateConfigSettings;
  }

  /** Returns the object with the settings used for calls to createGroup. */
  public UnaryCallSettings<CreateGroupRequest, Group> createGroupSettings() {
    return createGroupSettings;
  }

  /** Returns the object with the settings used for calls to getGroup. */
  public UnaryCallSettings<GetGroupRequest, Group> getGroupSettings() {
    return getGroupSettings;
  }

  /** Returns the object with the settings used for calls to listGroups. */
  public UnaryCallSettings<ListGroupsRequest, ListGroupsResponse> listGroupsSettings() {
    return listGroupsSettings;
  }

  /** Returns the object with the settings used for calls to deleteGroup. */
  public UnaryCallSettings<DeleteGroupRequest, DeleteGroupResponse> deleteGroupSettings() {
    return deleteGroupSettings;
  }

  /** Returns the object with the settings used for calls to updateGroup. */
  public UnaryCallSettings<UpdateGroupRequest, Group> updateGroupSettings() {
    return updateGroupSettings;
  }

  /** Returns the object with the settings used for calls to disableGroups. */
  public UnaryCallSettings<DisableGroupsRequest, DisableGroupsResponse> disableGroupsSettings() {
    return disableGroupsSettings;
  }

  /** Returns the object with the settings used for calls to enableGroups. */
  public UnaryCallSettings<EnableGroupsRequest, EnableGroupsResponse> enableGroupsSettings() {
    return enableGroupsSettings;
  }

  /** Returns the object with the settings used for calls to createGroupMember. */
  public UnaryCallSettings<CreateGroupMemberRequest, GroupMember> createGroupMemberSettings() {
    return createGroupMemberSettings;
  }

  /** Returns the object with the settings used for calls to createGroupMembers. */
  public UnaryCallSettings<CreateGroupMembersRequest, CreateGroupMembersResponse>
      createGroupMembersSettings() {
    return createGroupMembersSettings;
  }

  /** Returns the object with the settings used for calls to getGroupMember. */
  public UnaryCallSettings<GetGroupMemberRequest, GroupMember> getGroupMemberSettings() {
    return getGroupMemberSettings;
  }

  /** Returns the object with the settings used for calls to listGroupMembers. */
  public UnaryCallSettings<ListGroupMembersRequest, ListGroupMembersResponse>
      listGroupMembersSettings() {
    return listGroupMembersSettings;
  }

  /** Returns the object with the settings used for calls to deleteGroupMember. */
  public UnaryCallSettings<DeleteGroupMemberRequest, DeleteGroupMemberResponse>
      deleteGroupMemberSettings() {
    return deleteGroupMemberSettings;
  }

  /** Returns the object with the settings used for calls to deleteGroupMembers. */
  public UnaryCallSettings<DeleteGroupMembersRequest, DeleteGroupMembersResponse>
      deleteGroupMembersSettings() {
    return deleteGroupMembersSettings;
  }

  /** Returns the object with the settings used for calls to getGroupChart. */
  public UnaryCallSettings<GetGroupChartRequest, Chart> getGroupChartSettings() {
    return getGroupChartSettings;
  }

  /** Returns the object with the settings used for calls to listGroupCharts. */
  public UnaryCallSettings<ListGroupChartsRequest, ListGroupChartsResponse>
      listGroupChartsSettings() {
    return listGroupChartsSettings;
  }

  /** Returns the object with the settings used for calls to listGroupConfigs. */
  public UnaryCallSettings<ListGroupConfigsRequest, ListGroupConfigsResponse>
      listGroupConfigsSettings() {
    return listGroupConfigsSettings;
  }

  /** Returns the object with the settings used for calls to updateGroupConfigs. */
  public UnaryCallSettings<UpdateGroupConfigsRequest, UpdateGroupConfigsResponse>
      updateGroupConfigsSettings() {
    return updateGroupConfigsSettings;
  }

  /** Returns the object with the settings used for calls to disableAllEnabledGroupsOfConfig. */
  public UnaryCallSettings<
          DisableAllEnabledGroupsOfConfigRequest, DisableAllEnabledGroupsOfConfigResponse>
      disableAllEnabledGroupsOfConfigSettings() {
    return disableAllEnabledGroupsOfConfigSettings;
  }

  /** Returns the object with the settings used for calls to listConfigPackages. */
  public UnaryCallSettings<ListConfigPackagesRequest, ListConfigPackagesResponse>
      listConfigPackagesSettings() {
    return listConfigPackagesSettings;
  }

  /** Returns the object with the settings used for calls to updateConfigPackages. */
  public UnaryCallSettings<UpdateConfigPackagesRequest, UpdateConfigPackagesResponse>
      updateConfigPackagesSettings() {
    return updateConfigPackagesSettings;
  }

  /** Returns the object with the settings used for calls to listConfigsUsingPackage. */
  public UnaryCallSettings<ListConfigsUsingPackageRequest, ListConfigsUsingPackageResponse>
      listConfigsUsingPackageSettings() {
    return listConfigsUsingPackageSettings;
  }

  @BetaApi("A restructuring of stub classes is planned, so this may break in the future")
  public OverTheAirServiceStub createStub() throws IOException {
    if (getTransportChannelProvider()
        .getTransportName()
        .equals(GrpcTransportChannel.getGrpcTransportName())) {
      return GrpcOverTheAirServiceStub.create(this);
    } else {
      throw new UnsupportedOperationException(
          "Transport not supported: " + getTransportChannelProvider().getTransportName());
    }
  }

  /** Returns a builder for the default ExecutorProvider for this service. */
  public static InstantiatingExecutorProvider.Builder defaultExecutorProviderBuilder() {
    return InstantiatingExecutorProvider.newBuilder();
  }

  /** Returns the default service endpoint. */
  public static String getDefaultEndpoint() {
    return "androidovertheair.googleapis.com:443";
  }

  /** Returns the default service scopes. */
  public static List<String> getDefaultServiceScopes() {
    return DEFAULT_SERVICE_SCOPES;
  }

  /** Returns a builder for the default credentials for this service. */
  public static GoogleCredentialsProvider.Builder defaultCredentialsProviderBuilder() {
    return GoogleCredentialsProvider.newBuilder().setScopesToApply(DEFAULT_SERVICE_SCOPES);
  }

  /** Returns a builder for the default ChannelProvider for this service. */
  public static InstantiatingGrpcChannelProvider.Builder defaultGrpcTransportProviderBuilder() {
    return InstantiatingGrpcChannelProvider.newBuilder();
  }

  public static TransportChannelProvider defaultTransportChannelProvider() {
    return defaultGrpcTransportProviderBuilder().build();
  }

  @BetaApi("The surface for customizing headers is not stable yet and may change in the future.")
  public static ApiClientHeaderProvider.Builder defaultApiClientHeaderProviderBuilder() {
    return ApiClientHeaderProvider.newBuilder()
        .setGeneratedLibToken(
            "gapic", GaxProperties.getLibraryVersion(OverTheAirServiceStubSettings.class))
        .setTransportToken(
            GaxGrpcProperties.getGrpcTokenName(), GaxGrpcProperties.getGrpcVersion());
  }

  /** Returns a new builder for this class. */
  public static Builder newBuilder() {
    return Builder.createDefault();
  }

  /** Returns a new builder for this class. */
  public static Builder newBuilder(ClientContext clientContext) {
    return new Builder(clientContext);
  }

  /** Returns a builder containing all the values of this settings class. */
  public Builder toBuilder() {
    return new Builder(this);
  }

  protected OverTheAirServiceStubSettings(Builder settingsBuilder) throws IOException {
    super(settingsBuilder);

    getDeploymentSettings = settingsBuilder.getDeploymentSettings().build();
    listDeploymentsSettings = settingsBuilder.listDeploymentsSettings().build();
    updateDeploymentSettings = settingsBuilder.updateDeploymentSettings().build();
    getPackageSettings = settingsBuilder.getPackageSettings().build();
    listPackagesSettings = settingsBuilder.listPackagesSettings().build();
    deletePackageSettings = settingsBuilder.deletePackageSettings().build();
    updatePackageSettings = settingsBuilder.updatePackageSettings().build();
    createConfigSettings = settingsBuilder.createConfigSettings().build();
    getConfigSettings = settingsBuilder.getConfigSettings().build();
    listConfigsSettings = settingsBuilder.listConfigsSettings().build();
    deleteConfigSettings = settingsBuilder.deleteConfigSettings().build();
    updateConfigSettings = settingsBuilder.updateConfigSettings().build();
    createGroupSettings = settingsBuilder.createGroupSettings().build();
    getGroupSettings = settingsBuilder.getGroupSettings().build();
    listGroupsSettings = settingsBuilder.listGroupsSettings().build();
    deleteGroupSettings = settingsBuilder.deleteGroupSettings().build();
    updateGroupSettings = settingsBuilder.updateGroupSettings().build();
    disableGroupsSettings = settingsBuilder.disableGroupsSettings().build();
    enableGroupsSettings = settingsBuilder.enableGroupsSettings().build();
    createGroupMemberSettings = settingsBuilder.createGroupMemberSettings().build();
    createGroupMembersSettings = settingsBuilder.createGroupMembersSettings().build();
    getGroupMemberSettings = settingsBuilder.getGroupMemberSettings().build();
    listGroupMembersSettings = settingsBuilder.listGroupMembersSettings().build();
    deleteGroupMemberSettings = settingsBuilder.deleteGroupMemberSettings().build();
    deleteGroupMembersSettings = settingsBuilder.deleteGroupMembersSettings().build();
    getGroupChartSettings = settingsBuilder.getGroupChartSettings().build();
    listGroupChartsSettings = settingsBuilder.listGroupChartsSettings().build();
    listGroupConfigsSettings = settingsBuilder.listGroupConfigsSettings().build();
    updateGroupConfigsSettings = settingsBuilder.updateGroupConfigsSettings().build();
    disableAllEnabledGroupsOfConfigSettings =
        settingsBuilder.disableAllEnabledGroupsOfConfigSettings().build();
    listConfigPackagesSettings = settingsBuilder.listConfigPackagesSettings().build();
    updateConfigPackagesSettings = settingsBuilder.updateConfigPackagesSettings().build();
    listConfigsUsingPackageSettings = settingsBuilder.listConfigsUsingPackageSettings().build();
  }

  /** Builder for OverTheAirServiceStubSettings. */
  public static class Builder extends StubSettings.Builder<OverTheAirServiceStubSettings, Builder> {
    private final ImmutableList<UnaryCallSettings.Builder<?, ?>> unaryMethodSettingsBuilders;

    private final UnaryCallSettings.Builder<GetDeploymentRequest, Deployment> getDeploymentSettings;
    private final UnaryCallSettings.Builder<ListDeploymentsRequest, ListDeploymentsResponse>
        listDeploymentsSettings;
    private final UnaryCallSettings.Builder<UpdateDeploymentRequest, Deployment>
        updateDeploymentSettings;
    private final UnaryCallSettings.Builder<
            GetPackageRequest, com.google.android.overtheair.v1.Package>
        getPackageSettings;
    private final UnaryCallSettings.Builder<ListPackagesRequest, ListPackagesResponse>
        listPackagesSettings;
    private final UnaryCallSettings.Builder<DeletePackageRequest, DeletePackageResponse>
        deletePackageSettings;
    private final UnaryCallSettings.Builder<
            UpdatePackageRequest, com.google.android.overtheair.v1.Package>
        updatePackageSettings;
    private final UnaryCallSettings.Builder<CreateConfigRequest, Config> createConfigSettings;
    private final UnaryCallSettings.Builder<GetConfigRequest, Config> getConfigSettings;
    private final UnaryCallSettings.Builder<ListConfigsRequest, ListConfigsResponse>
        listConfigsSettings;
    private final UnaryCallSettings.Builder<DeleteConfigRequest, DeleteConfigResponse>
        deleteConfigSettings;
    private final UnaryCallSettings.Builder<UpdateConfigRequest, Config> updateConfigSettings;
    private final UnaryCallSettings.Builder<CreateGroupRequest, Group> createGroupSettings;
    private final UnaryCallSettings.Builder<GetGroupRequest, Group> getGroupSettings;
    private final UnaryCallSettings.Builder<ListGroupsRequest, ListGroupsResponse>
        listGroupsSettings;
    private final UnaryCallSettings.Builder<DeleteGroupRequest, DeleteGroupResponse>
        deleteGroupSettings;
    private final UnaryCallSettings.Builder<UpdateGroupRequest, Group> updateGroupSettings;
    private final UnaryCallSettings.Builder<DisableGroupsRequest, DisableGroupsResponse>
        disableGroupsSettings;
    private final UnaryCallSettings.Builder<EnableGroupsRequest, EnableGroupsResponse>
        enableGroupsSettings;
    private final UnaryCallSettings.Builder<CreateGroupMemberRequest, GroupMember>
        createGroupMemberSettings;
    private final UnaryCallSettings.Builder<CreateGroupMembersRequest, CreateGroupMembersResponse>
        createGroupMembersSettings;
    private final UnaryCallSettings.Builder<GetGroupMemberRequest, GroupMember>
        getGroupMemberSettings;
    private final UnaryCallSettings.Builder<ListGroupMembersRequest, ListGroupMembersResponse>
        listGroupMembersSettings;
    private final UnaryCallSettings.Builder<DeleteGroupMemberRequest, DeleteGroupMemberResponse>
        deleteGroupMemberSettings;
    private final UnaryCallSettings.Builder<DeleteGroupMembersRequest, DeleteGroupMembersResponse>
        deleteGroupMembersSettings;
    private final UnaryCallSettings.Builder<GetGroupChartRequest, Chart> getGroupChartSettings;
    private final UnaryCallSettings.Builder<ListGroupChartsRequest, ListGroupChartsResponse>
        listGroupChartsSettings;
    private final UnaryCallSettings.Builder<ListGroupConfigsRequest, ListGroupConfigsResponse>
        listGroupConfigsSettings;
    private final UnaryCallSettings.Builder<UpdateGroupConfigsRequest, UpdateGroupConfigsResponse>
        updateGroupConfigsSettings;
    private final UnaryCallSettings.Builder<
            DisableAllEnabledGroupsOfConfigRequest, DisableAllEnabledGroupsOfConfigResponse>
        disableAllEnabledGroupsOfConfigSettings;
    private final UnaryCallSettings.Builder<ListConfigPackagesRequest, ListConfigPackagesResponse>
        listConfigPackagesSettings;
    private final UnaryCallSettings.Builder<
            UpdateConfigPackagesRequest, UpdateConfigPackagesResponse>
        updateConfigPackagesSettings;
    private final UnaryCallSettings.Builder<
            ListConfigsUsingPackageRequest, ListConfigsUsingPackageResponse>
        listConfigsUsingPackageSettings;

    private static final ImmutableMap<String, ImmutableSet<StatusCode.Code>>
        RETRYABLE_CODE_DEFINITIONS;

    static {
      ImmutableMap.Builder<String, ImmutableSet<StatusCode.Code>> definitions =
          ImmutableMap.builder();
      definitions.put(
          "idempotent",
          ImmutableSet.copyOf(
              Lists.<StatusCode.Code>newArrayList(
                  StatusCode.Code.DEADLINE_EXCEEDED, StatusCode.Code.UNAVAILABLE)));
      definitions.put("non_idempotent", ImmutableSet.copyOf(Lists.<StatusCode.Code>newArrayList()));
      RETRYABLE_CODE_DEFINITIONS = definitions.build();
    }

    private static final ImmutableMap<String, RetrySettings> RETRY_PARAM_DEFINITIONS;

    static {
      ImmutableMap.Builder<String, RetrySettings> definitions = ImmutableMap.builder();
      RetrySettings settings = null;
      settings =
          RetrySettings.newBuilder()
              .setInitialRetryDelay(Duration.ofMillis(100L))
              .setRetryDelayMultiplier(1.3)
              .setMaxRetryDelay(Duration.ofMillis(60000L))
              .setInitialRpcTimeout(Duration.ofMillis(20000L))
              .setRpcTimeoutMultiplier(1.0)
              .setMaxRpcTimeout(Duration.ofMillis(20000L))
              .setTotalTimeout(Duration.ofMillis(600000L))
              .build();
      definitions.put("default", settings);
      RETRY_PARAM_DEFINITIONS = definitions.build();
    }

    protected Builder() {
      this((ClientContext) null);
    }

    protected Builder(ClientContext clientContext) {
      super(clientContext);

      getDeploymentSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      listDeploymentsSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      updateDeploymentSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      getPackageSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      listPackagesSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      deletePackageSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      updatePackageSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      createConfigSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      getConfigSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      listConfigsSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      deleteConfigSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      updateConfigSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      createGroupSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      getGroupSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      listGroupsSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      deleteGroupSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      updateGroupSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      disableGroupsSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      enableGroupsSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      createGroupMemberSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      createGroupMembersSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      getGroupMemberSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      listGroupMembersSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      deleteGroupMemberSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      deleteGroupMembersSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      getGroupChartSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      listGroupChartsSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      listGroupConfigsSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      updateGroupConfigsSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      disableAllEnabledGroupsOfConfigSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      listConfigPackagesSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      updateConfigPackagesSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      listConfigsUsingPackageSettings = UnaryCallSettings.newUnaryCallSettingsBuilder();

      unaryMethodSettingsBuilders =
          ImmutableList.<UnaryCallSettings.Builder<?, ?>>of(
              getDeploymentSettings,
              listDeploymentsSettings,
              updateDeploymentSettings,
              getPackageSettings,
              listPackagesSettings,
              deletePackageSettings,
              updatePackageSettings,
              createConfigSettings,
              getConfigSettings,
              listConfigsSettings,
              deleteConfigSettings,
              updateConfigSettings,
              createGroupSettings,
              getGroupSettings,
              listGroupsSettings,
              deleteGroupSettings,
              updateGroupSettings,
              disableGroupsSettings,
              enableGroupsSettings,
              createGroupMemberSettings,
              createGroupMembersSettings,
              getGroupMemberSettings,
              listGroupMembersSettings,
              deleteGroupMemberSettings,
              deleteGroupMembersSettings,
              getGroupChartSettings,
              listGroupChartsSettings,
              listGroupConfigsSettings,
              updateGroupConfigsSettings,
              disableAllEnabledGroupsOfConfigSettings,
              listConfigPackagesSettings,
              updateConfigPackagesSettings,
              listConfigsUsingPackageSettings);

      initDefaults(this);
    }

    private static Builder createDefault() {
      Builder builder = new Builder((ClientContext) null);
      builder.setTransportChannelProvider(defaultTransportChannelProvider());
      builder.setCredentialsProvider(defaultCredentialsProviderBuilder().build());
      builder.setInternalHeaderProvider(defaultApiClientHeaderProviderBuilder().build());
      builder.setEndpoint(getDefaultEndpoint());
      return initDefaults(builder);
    }

    private static Builder initDefaults(Builder builder) {

      builder
          .getDeploymentSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .listDeploymentsSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .updateDeploymentSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .getPackageSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .listPackagesSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .deletePackageSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .updatePackageSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .createConfigSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("non_idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .getConfigSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .listConfigsSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .deleteConfigSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .updateConfigSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .createGroupSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("non_idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .getGroupSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .listGroupsSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .deleteGroupSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .updateGroupSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .disableGroupsSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("non_idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .enableGroupsSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("non_idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .createGroupMemberSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("non_idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .createGroupMembersSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("non_idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .getGroupMemberSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .listGroupMembersSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .deleteGroupMemberSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .deleteGroupMembersSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("non_idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .getGroupChartSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .listGroupChartsSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .listGroupConfigsSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .updateGroupConfigsSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .disableAllEnabledGroupsOfConfigSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("non_idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .listConfigPackagesSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .updateConfigPackagesSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      builder
          .listConfigsUsingPackageSettings()
          .setRetryableCodes(RETRYABLE_CODE_DEFINITIONS.get("idempotent"))
          .setRetrySettings(RETRY_PARAM_DEFINITIONS.get("default"));

      return builder;
    }

    protected Builder(OverTheAirServiceStubSettings settings) {
      super(settings);

      getDeploymentSettings = settings.getDeploymentSettings.toBuilder();
      listDeploymentsSettings = settings.listDeploymentsSettings.toBuilder();
      updateDeploymentSettings = settings.updateDeploymentSettings.toBuilder();
      getPackageSettings = settings.getPackageSettings.toBuilder();
      listPackagesSettings = settings.listPackagesSettings.toBuilder();
      deletePackageSettings = settings.deletePackageSettings.toBuilder();
      updatePackageSettings = settings.updatePackageSettings.toBuilder();
      createConfigSettings = settings.createConfigSettings.toBuilder();
      getConfigSettings = settings.getConfigSettings.toBuilder();
      listConfigsSettings = settings.listConfigsSettings.toBuilder();
      deleteConfigSettings = settings.deleteConfigSettings.toBuilder();
      updateConfigSettings = settings.updateConfigSettings.toBuilder();
      createGroupSettings = settings.createGroupSettings.toBuilder();
      getGroupSettings = settings.getGroupSettings.toBuilder();
      listGroupsSettings = settings.listGroupsSettings.toBuilder();
      deleteGroupSettings = settings.deleteGroupSettings.toBuilder();
      updateGroupSettings = settings.updateGroupSettings.toBuilder();
      disableGroupsSettings = settings.disableGroupsSettings.toBuilder();
      enableGroupsSettings = settings.enableGroupsSettings.toBuilder();
      createGroupMemberSettings = settings.createGroupMemberSettings.toBuilder();
      createGroupMembersSettings = settings.createGroupMembersSettings.toBuilder();
      getGroupMemberSettings = settings.getGroupMemberSettings.toBuilder();
      listGroupMembersSettings = settings.listGroupMembersSettings.toBuilder();
      deleteGroupMemberSettings = settings.deleteGroupMemberSettings.toBuilder();
      deleteGroupMembersSettings = settings.deleteGroupMembersSettings.toBuilder();
      getGroupChartSettings = settings.getGroupChartSettings.toBuilder();
      listGroupChartsSettings = settings.listGroupChartsSettings.toBuilder();
      listGroupConfigsSettings = settings.listGroupConfigsSettings.toBuilder();
      updateGroupConfigsSettings = settings.updateGroupConfigsSettings.toBuilder();
      disableAllEnabledGroupsOfConfigSettings =
          settings.disableAllEnabledGroupsOfConfigSettings.toBuilder();
      listConfigPackagesSettings = settings.listConfigPackagesSettings.toBuilder();
      updateConfigPackagesSettings = settings.updateConfigPackagesSettings.toBuilder();
      listConfigsUsingPackageSettings = settings.listConfigsUsingPackageSettings.toBuilder();

      unaryMethodSettingsBuilders =
          ImmutableList.<UnaryCallSettings.Builder<?, ?>>of(
              getDeploymentSettings,
              listDeploymentsSettings,
              updateDeploymentSettings,
              getPackageSettings,
              listPackagesSettings,
              deletePackageSettings,
              updatePackageSettings,
              createConfigSettings,
              getConfigSettings,
              listConfigsSettings,
              deleteConfigSettings,
              updateConfigSettings,
              createGroupSettings,
              getGroupSettings,
              listGroupsSettings,
              deleteGroupSettings,
              updateGroupSettings,
              disableGroupsSettings,
              enableGroupsSettings,
              createGroupMemberSettings,
              createGroupMembersSettings,
              getGroupMemberSettings,
              listGroupMembersSettings,
              deleteGroupMemberSettings,
              deleteGroupMembersSettings,
              getGroupChartSettings,
              listGroupChartsSettings,
              listGroupConfigsSettings,
              updateGroupConfigsSettings,
              disableAllEnabledGroupsOfConfigSettings,
              listConfigPackagesSettings,
              updateConfigPackagesSettings,
              listConfigsUsingPackageSettings);
    }

    // NEXT_MAJOR_VER: remove 'throws Exception'
    /**
     * Applies the given settings updater function to all of the unary API methods in this service.
     *
     * <p>Note: This method does not support applying settings to streaming methods.
     */
    public Builder applyToAllUnaryMethods(
        ApiFunction<UnaryCallSettings.Builder<?, ?>, Void> settingsUpdater) throws Exception {
      super.applyToAllUnaryMethods(unaryMethodSettingsBuilders, settingsUpdater);
      return this;
    }

    public ImmutableList<UnaryCallSettings.Builder<?, ?>> unaryMethodSettingsBuilders() {
      return unaryMethodSettingsBuilders;
    }

    /** Returns the builder for the settings used for calls to getDeployment. */
    public UnaryCallSettings.Builder<GetDeploymentRequest, Deployment> getDeploymentSettings() {
      return getDeploymentSettings;
    }

    /** Returns the builder for the settings used for calls to listDeployments. */
    public UnaryCallSettings.Builder<ListDeploymentsRequest, ListDeploymentsResponse>
        listDeploymentsSettings() {
      return listDeploymentsSettings;
    }

    /** Returns the builder for the settings used for calls to updateDeployment. */
    public UnaryCallSettings.Builder<UpdateDeploymentRequest, Deployment>
        updateDeploymentSettings() {
      return updateDeploymentSettings;
    }

    /** Returns the builder for the settings used for calls to getPackage. */
    public UnaryCallSettings.Builder<GetPackageRequest, com.google.android.overtheair.v1.Package>
        getPackageSettings() {
      return getPackageSettings;
    }

    /** Returns the builder for the settings used for calls to listPackages. */
    public UnaryCallSettings.Builder<ListPackagesRequest, ListPackagesResponse>
        listPackagesSettings() {
      return listPackagesSettings;
    }

    /** Returns the builder for the settings used for calls to deletePackage. */
    public UnaryCallSettings.Builder<DeletePackageRequest, DeletePackageResponse>
        deletePackageSettings() {
      return deletePackageSettings;
    }

    /** Returns the builder for the settings used for calls to updatePackage. */
    public UnaryCallSettings.Builder<UpdatePackageRequest, com.google.android.overtheair.v1.Package>
        updatePackageSettings() {
      return updatePackageSettings;
    }

    /** Returns the builder for the settings used for calls to createConfig. */
    public UnaryCallSettings.Builder<CreateConfigRequest, Config> createConfigSettings() {
      return createConfigSettings;
    }

    /** Returns the builder for the settings used for calls to getConfig. */
    public UnaryCallSettings.Builder<GetConfigRequest, Config> getConfigSettings() {
      return getConfigSettings;
    }

    /** Returns the builder for the settings used for calls to listConfigs. */
    public UnaryCallSettings.Builder<ListConfigsRequest, ListConfigsResponse>
        listConfigsSettings() {
      return listConfigsSettings;
    }

    /** Returns the builder for the settings used for calls to deleteConfig. */
    public UnaryCallSettings.Builder<DeleteConfigRequest, DeleteConfigResponse>
        deleteConfigSettings() {
      return deleteConfigSettings;
    }

    /** Returns the builder for the settings used for calls to updateConfig. */
    public UnaryCallSettings.Builder<UpdateConfigRequest, Config> updateConfigSettings() {
      return updateConfigSettings;
    }

    /** Returns the builder for the settings used for calls to createGroup. */
    public UnaryCallSettings.Builder<CreateGroupRequest, Group> createGroupSettings() {
      return createGroupSettings;
    }

    /** Returns the builder for the settings used for calls to getGroup. */
    public UnaryCallSettings.Builder<GetGroupRequest, Group> getGroupSettings() {
      return getGroupSettings;
    }

    /** Returns the builder for the settings used for calls to listGroups. */
    public UnaryCallSettings.Builder<ListGroupsRequest, ListGroupsResponse> listGroupsSettings() {
      return listGroupsSettings;
    }

    /** Returns the builder for the settings used for calls to deleteGroup. */
    public UnaryCallSettings.Builder<DeleteGroupRequest, DeleteGroupResponse>
        deleteGroupSettings() {
      return deleteGroupSettings;
    }

    /** Returns the builder for the settings used for calls to updateGroup. */
    public UnaryCallSettings.Builder<UpdateGroupRequest, Group> updateGroupSettings() {
      return updateGroupSettings;
    }

    /** Returns the builder for the settings used for calls to disableGroups. */
    public UnaryCallSettings.Builder<DisableGroupsRequest, DisableGroupsResponse>
        disableGroupsSettings() {
      return disableGroupsSettings;
    }

    /** Returns the builder for the settings used for calls to enableGroups. */
    public UnaryCallSettings.Builder<EnableGroupsRequest, EnableGroupsResponse>
        enableGroupsSettings() {
      return enableGroupsSettings;
    }

    /** Returns the builder for the settings used for calls to createGroupMember. */
    public UnaryCallSettings.Builder<CreateGroupMemberRequest, GroupMember>
        createGroupMemberSettings() {
      return createGroupMemberSettings;
    }

    /** Returns the builder for the settings used for calls to createGroupMembers. */
    public UnaryCallSettings.Builder<CreateGroupMembersRequest, CreateGroupMembersResponse>
        createGroupMembersSettings() {
      return createGroupMembersSettings;
    }

    /** Returns the builder for the settings used for calls to getGroupMember. */
    public UnaryCallSettings.Builder<GetGroupMemberRequest, GroupMember> getGroupMemberSettings() {
      return getGroupMemberSettings;
    }

    /** Returns the builder for the settings used for calls to listGroupMembers. */
    public UnaryCallSettings.Builder<ListGroupMembersRequest, ListGroupMembersResponse>
        listGroupMembersSettings() {
      return listGroupMembersSettings;
    }

    /** Returns the builder for the settings used for calls to deleteGroupMember. */
    public UnaryCallSettings.Builder<DeleteGroupMemberRequest, DeleteGroupMemberResponse>
        deleteGroupMemberSettings() {
      return deleteGroupMemberSettings;
    }

    /** Returns the builder for the settings used for calls to deleteGroupMembers. */
    public UnaryCallSettings.Builder<DeleteGroupMembersRequest, DeleteGroupMembersResponse>
        deleteGroupMembersSettings() {
      return deleteGroupMembersSettings;
    }

    /** Returns the builder for the settings used for calls to getGroupChart. */
    public UnaryCallSettings.Builder<GetGroupChartRequest, Chart> getGroupChartSettings() {
      return getGroupChartSettings;
    }

    /** Returns the builder for the settings used for calls to listGroupCharts. */
    public UnaryCallSettings.Builder<ListGroupChartsRequest, ListGroupChartsResponse>
        listGroupChartsSettings() {
      return listGroupChartsSettings;
    }

    /** Returns the builder for the settings used for calls to listGroupConfigs. */
    public UnaryCallSettings.Builder<ListGroupConfigsRequest, ListGroupConfigsResponse>
        listGroupConfigsSettings() {
      return listGroupConfigsSettings;
    }

    /** Returns the builder for the settings used for calls to updateGroupConfigs. */
    public UnaryCallSettings.Builder<UpdateGroupConfigsRequest, UpdateGroupConfigsResponse>
        updateGroupConfigsSettings() {
      return updateGroupConfigsSettings;
    }

    /** Returns the builder for the settings used for calls to disableAllEnabledGroupsOfConfig. */
    public UnaryCallSettings.Builder<
            DisableAllEnabledGroupsOfConfigRequest, DisableAllEnabledGroupsOfConfigResponse>
        disableAllEnabledGroupsOfConfigSettings() {
      return disableAllEnabledGroupsOfConfigSettings;
    }

    /** Returns the builder for the settings used for calls to listConfigPackages. */
    public UnaryCallSettings.Builder<ListConfigPackagesRequest, ListConfigPackagesResponse>
        listConfigPackagesSettings() {
      return listConfigPackagesSettings;
    }

    /** Returns the builder for the settings used for calls to updateConfigPackages. */
    public UnaryCallSettings.Builder<UpdateConfigPackagesRequest, UpdateConfigPackagesResponse>
        updateConfigPackagesSettings() {
      return updateConfigPackagesSettings;
    }

    /** Returns the builder for the settings used for calls to listConfigsUsingPackage. */
    public UnaryCallSettings.Builder<
            ListConfigsUsingPackageRequest, ListConfigsUsingPackageResponse>
        listConfigsUsingPackageSettings() {
      return listConfigsUsingPackageSettings;
    }

    @Override
    public OverTheAirServiceStubSettings build() throws IOException {
      return new OverTheAirServiceStubSettings(this);
    }
  }
}
