/*
 * Copyright 2018 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.google.android.overtheair.v1;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableMap;
import com.google.api.pathtemplate.PathTemplate;
import com.google.api.resourcenames.ResourceName;
import java.util.Map;
import java.util.ArrayList;
import java.util.List;

// AUTO-GENERATED DOCUMENTATION AND CLASS
@javax.annotation.Generated("by GAPIC protoc plugin")
public class ChartName implements ResourceName {

  private static final PathTemplate PATH_TEMPLATE =
      PathTemplate.createWithoutUrlEncoding("deployments/{deployment}/groups/{group}/charts/{chart}");

  private volatile Map<String, String> fieldValuesMap;

  private final String deployment;
  private final String group;
  private final String chart;

  public String getDeployment() {
    return deployment;
  }

  public String getGroup() {
    return group;
  }

  public String getChart() {
    return chart;
  }

  public static Builder newBuilder() {
    return new Builder();
  }

  public Builder toBuilder() {
    return new Builder(this);
  }

  private ChartName(Builder builder) {
    deployment = Preconditions.checkNotNull(builder.getDeployment());
    group = Preconditions.checkNotNull(builder.getGroup());
    chart = Preconditions.checkNotNull(builder.getChart());
  }

  public static ChartName of(String deployment, String group, String chart) {
    return newBuilder()
      .setDeployment(deployment)
      .setGroup(group)
      .setChart(chart)
      .build();
  }

  public static String format(String deployment, String group, String chart) {
    return newBuilder()
      .setDeployment(deployment)
      .setGroup(group)
      .setChart(chart)
      .build()
      .toString();
  }

  public static ChartName parse(String formattedString) {
    if (formattedString.isEmpty()) {
      return null;
    }
    Map<String, String> matchMap =
        PATH_TEMPLATE.validatedMatch(formattedString, "ChartName.parse: formattedString not in valid format");
    return of(matchMap.get("deployment"), matchMap.get("group"), matchMap.get("chart"));
  }

  public static List<ChartName> parseList(List<String> formattedStrings) {
    List<ChartName> list = new ArrayList<>(formattedStrings.size());
    for (String formattedString : formattedStrings) {
      list.add(parse(formattedString));
    }
    return list;
  }

  public static List<String> toStringList(List<ChartName> values) {
    List<String> list = new ArrayList<String>(values.size());
    for (ChartName value : values) {
      if (value == null) {
        list.add("");
      } else {
        list.add(value.toString());
      }
    }
    return list;
  }

  public static boolean isParsableFrom(String formattedString) {
    return PATH_TEMPLATE.matches(formattedString);
  }

  public Map<String, String> getFieldValuesMap() {
    if (fieldValuesMap == null) {
      synchronized (this) {
        if (fieldValuesMap == null) {
          ImmutableMap.Builder<String, String> fieldMapBuilder = ImmutableMap.builder();
          fieldMapBuilder.put("deployment", deployment);
          fieldMapBuilder.put("group", group);
          fieldMapBuilder.put("chart", chart);
          fieldValuesMap = fieldMapBuilder.build();
        }
      }
    }
    return fieldValuesMap;
  }

  public String getFieldValue(String fieldName) {
    return getFieldValuesMap().get(fieldName);
  }

  @Override
  public String toString() {
    return PATH_TEMPLATE.instantiate("deployment", deployment, "group", group, "chart", chart);
  }

  /** Builder for ChartName. */
  public static class Builder {

    private String deployment;
    private String group;
    private String chart;

    public String getDeployment() {
      return deployment;
    }

    public String getGroup() {
      return group;
    }

    public String getChart() {
      return chart;
    }

    public Builder setDeployment(String deployment) {
      this.deployment = deployment;
      return this;
    }

    public Builder setGroup(String group) {
      this.group = group;
      return this;
    }

    public Builder setChart(String chart) {
      this.chart = chart;
      return this;
    }

    private Builder() {
    }

    private Builder(ChartName chartName) {
      deployment = chartName.deployment;
      group = chartName.group;
      chart = chartName.chart;
    }

    public ChartName build() {
      return new ChartName(this);
    }
  }

  @Override
  public boolean equals(Object o) {
    if (o == this) {
      return true;
    }
    if (o instanceof ChartName) {
      ChartName that = (ChartName) o;
      return (this.deployment.equals(that.deployment))
          && (this.group.equals(that.group))
          && (this.chart.equals(that.chart));
    }
    return false;
  }

  @Override
  public int hashCode() {
    int h = 1;
    h *= 1000003;
    h ^= deployment.hashCode();
    h *= 1000003;
    h ^= group.hashCode();
    h *= 1000003;
    h ^= chart.hashCode();
    return h;
  }
}

